package no.ntnu.idatt1002.demo.data.Budget;

import no.ntnu.idatt1002.demo.data.Economics.ExpenseCategory;

import java.sql.Date;
import java.time.Duration;
import java.time.LocalDate;
import java.time.ZoneId;
import java.time.temporal.ChronoUnit;
import java.time.temporal.Temporal;
import java.util.Calendar;
import java.util.List;

/**
 * Class that represents GeneralBudget and implements the Budget interface.
 *
 * @author Adele
 */
public class GeneralBudget {

    private int budgetPeriod;
    private List<BudgetItem> listOfItems;
    private double maxAmount;

    /**
     * The constructor of a new GeneralBudget.
     *
     * @param budgetPeriod The period the budget are going to last for
     *
     */
    public GeneralBudget(int budgetPeriod, List<BudgetItem> listOfItems, double maxAmount){
        this.budgetPeriod = budgetPeriod;
        this.maxAmount = maxAmount;
        this.listOfItems = listOfItems;

    }

    /**
     * This method returns the amount of days the budget should last for.
     *
     * @return amount of days the budget lasts
     *
     */
    public int getBudgetPeriod() {
        return budgetPeriod;
    }

    /**
     * This method returns the amount of days left in the period as a string.
     *
     * @return the amount of days left in the budget period
     *
     */
    public long getDaysLeftOfBudgetPeriod() {
        LocalDate today = LocalDate.now();
        LocalDate end = today.plus(Duration.ofDays(getBudgetPeriod()));

        Date todaysDate = (Date) Date.from(today.atStartOfDay(ZoneId.systemDefault()).toInstant());
        Date endDate = (Date) Date.from(end.atStartOfDay(ZoneId.systemDefault()).toInstant());

        Calendar cStart  = Calendar.getInstance(); cStart.setTime(todaysDate);
        Calendar cEnd = Calendar.getInstance(); cEnd.setTime(endDate);

        return ChronoUnit.DAYS.between((Temporal) cStart, (Temporal) cEnd);
    }

    public void addToBudget(double budgetAmount, String description, ExpenseCategory category) {
        if (totalSum() + budgetAmount > maxAmount ){
            throw new IllegalArgumentException("Amount to be added goes over the max value of the budget");
        }

        if (!checksListOfItemsContainsBudgetItem(category)) {
            listOfItems.add(new BudgetItem(budgetAmount, description, category));
        } else {
            throw new IllegalArgumentException("There is already a budget with this category");
        }
    }

    public boolean checksListOfItemsContainsBudgetItem(ExpenseCategory category){
        for (BudgetItem item : listOfItems) {
            if (item.getCategory() == category) {
                return true;
            }
        }
        return false;
    }

    public double totalSum(){
        double sum = 0;
        for (BudgetItem budgetItem : listOfItems) {
            sum += budgetItem.getBudgetAmount();
        }
        return sum;
    }

    public void deleteItemFromBudget(ExpenseCategory category){
        listOfItems.removeIf(item -> item.getCategory() == category);
    }
}
