package no.ntnu.idatt1002.demo.data.Economics;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.*;


/**
 * FileHandling is a class for writing and reading
 * Item-objects to and from a file.
 */
public class FileHandling {
    /**
     * Method for writing (adding) an ItemRegister to a file.
     *
     * @param itemRegister the ItemRegister you want to write to a file.
     * @throws IOException if an input or output exception occurred.
     */
    public <T extends Item>void writeItemRegisterToFile(final ItemRegister<T> itemRegister, String fileTitle) throws IOException {
        try (BufferedWriter bw = new BufferedWriter(new FileWriter("src/main/resources/" + fileTitle + ".itemRegister"))) {
            bw.write(itemRegister.toString());
        } catch (IOException ex) {
            throw new IOException("Error writing story to file: " + ex.getMessage());
        }
    }

    public ItemRegister<Income> readIncomeRegisterFromFile(String fileTitle) throws IOException {
        IncomeRegister incomeRegister = new IncomeRegister();
        String date = "";
        String description = "";
        double amount = 0;
        boolean reoccuring = false;
        IncomeCategory incomeCategory = null;
        try (BufferedReader br = new BufferedReader(new FileReader("src/main/resources/" + fileTitle + ".itemRegister"))) {
            br.readLine();
            String line;
            String nextLine = br.readLine();
            while ((line = nextLine) != null) {
                nextLine = br.readLine();
                if(line.startsWith("date=")) {
                    date = line.replace("date=", "");
                } else if (line.startsWith("description=")) {
                    description = line.replace("description=","");
                } else if (line.startsWith("amount=")) {
                    amount = Double.parseDouble(line.replace("amount=",""));
                } else if (line.startsWith("isReoccuring=")) {
                    reoccuring = line.replace("isReoccuring=","").equals("Reoccurring");
                } else if (line.startsWith("category=")) {
                    line = line.replace("category=","");
                    incomeCategory = switch (line) {
                        case "GIFT" -> IncomeCategory.GIFT;
                        case "STUDENT_LOAN" -> IncomeCategory.STUDENT_LOAN;
                        default -> IncomeCategory.SALARY;
                    };
                }
                if(line.isEmpty() || (nextLine == null)) {
                    if(description.equals("")){
                        incomeRegister.addItem(new Income(amount, reoccuring, incomeCategory, date));
                    } else{
                        incomeRegister.addItem(new Income(description, amount, reoccuring, incomeCategory, date));
                    }
                    description = "";
                }
            }
        }
        return incomeRegister;
    }

    public ItemRegister<Expense> readExpenseRegisterFromFile(String fileTitle) throws IOException {
        ExpenseRegister expenseRegister = new ExpenseRegister();
        String date = "";
        String description = "";
        double amount = 0;
        boolean reoccuring = false;
        ExpenseCategory expenseCategory = null;
        try (BufferedReader br = new BufferedReader(new FileReader("src/main/resources/" + fileTitle + ".itemRegister"))) {
            br.readLine();
            String line;
            String nextLine = br.readLine();
            while ((line = nextLine) != null) {
                nextLine = br.readLine();
                if (line.startsWith("date=")) {
                    date = line.replace("date=", "");
                } else if (line.startsWith("description=")) {
                    description = line.replace("description=", "");
                } else if (line.startsWith("amount=")) {
                    amount = Double.parseDouble(line.replace("amount=", ""));
                } else if (line.startsWith("isReoccuring=")) {
                    reoccuring = line.replace("isReoccuring=", "").equals("Reoccurring");
                } else if (line.startsWith("category=")) {
                    line = line.replace("category=", "");
                    expenseCategory = switch (line) {
                        case "FOOD" -> ExpenseCategory.FOOD;
                        case "CLOTHES" -> ExpenseCategory.CLOTHES;
                        case "BOOKS" -> ExpenseCategory.BOOKS;
                        default -> ExpenseCategory.OTHER;
                    };
                }
                if (line.isEmpty() || (nextLine == null)) {
                    if (description.equals("")) {
                        expenseRegister.addItem(new Expense(amount, reoccuring, expenseCategory, date));
                    } else {
                        expenseRegister.addItem(new Expense(description, amount, reoccuring, expenseCategory, date));
                    }
                    description = "";
                }
            }
        }
        return expenseRegister;
    }
}