package no.ntnu.idatt1002.demo.data.Economics;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;

import java.util.List;

import static org.junit.jupiter.api.Assertions.*;
import static org.junit.jupiter.api.Assertions.assertThrows;

class ExpenseRegisterTest {
    ExpenseRegister expenseRegister = new ExpenseRegister();
    @Test
    @DisplayName("addItem method throws exception when it should")
    void addItemThrows() {
        Expense expense = new Expense("description", 59.9f, false, ExpenseCategory.CLOTHES, "03.03.23");
        expenseRegister.addItem(expense);
        assertThrows(IllegalArgumentException.class, () -> expenseRegister.addItem(expense));
    }

    @Test
    @DisplayName("addItem method does not throw exception when it should not")
    void addItemDoesNotThrow() {
        Expense expense1 = new Expense("description", 59.9f, false, ExpenseCategory.CLOTHES, "03.03.23");
        Expense expense2 = new Expense("anotherDescription", 6.5f, true, ExpenseCategory.BOOKS, "02.03.23");
        assertDoesNotThrow(() -> expenseRegister.addItem(expense1));
        assertDoesNotThrow(() -> expenseRegister.addItem(expense2));
    }

    @Nested
    @DisplayName("Test getTotalSum and getExpenseByCategory methods")
    class testGetExpenseByCategoryMethod {
        Expense expense1;
        Expense expense2;
        Expense expense3;

        @BeforeEach
        void addExpensesToRegister() {
            expense1 = new Expense("description1", 59.9f, false, ExpenseCategory.CLOTHES, "03.03.23");
            expense2 = new Expense("description2", 62.4f, true, ExpenseCategory.FOOD, "01.02.21");
            expense3 = new Expense("description3", 9.81f, false, ExpenseCategory.CLOTHES, "05.07.23");

            expenseRegister.addItem(expense1);
            expenseRegister.addItem(expense2);
            expenseRegister.addItem(expense3);
        }

        @Test
        @DisplayName("getTotalSum method gives correct amount")
        void getTotalSumCorrectAmount() {
            double totalIncome = 59.9f + 62.4f + 9.81f;
            assertEquals(Math.round(expenseRegister.getTotalSum()), Math.round(totalIncome));
        }

        @Test
        @DisplayName("getExpenseByCategory gives expected Expenses back")
        void getExpensesByCategoryGivesExpectedExpensesBack() {
            List<Expense> expenseSalary = expenseRegister.getExpenseByCategory(ExpenseCategory.CLOTHES);
            assertTrue(expenseSalary.contains(expense1) && expenseSalary.contains(expense3));
        }
    }
}
