package no.ntnu.idatt1002.demo.data.Economics;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;

import java.util.List;

import static org.junit.jupiter.api.Assertions.*;

public class IncomeRegisterTest {
    IncomeRegister incomeRegister = new IncomeRegister();
    @Test
    @DisplayName("addItem method throws exception when it should")
    void addItemThrows() {
        Income income = new Income("description", 59.9f, false, IncomeCategory.SALARY, "03.03.23");
        incomeRegister.addItem(income);
        assertThrows(IllegalArgumentException.class, () -> incomeRegister.addItem(income));
    }

    @Test
    @DisplayName("addItem method does not throw exception when it should not")
    void addItemDoesNotThrow() {
        Income income1 = new Income("description", 59.9f, false, IncomeCategory.GIFT, "03.03.23");
        Income income2 = new Income("anotherDescription", 6.5f, true, IncomeCategory.SALARY, "02.03.23");
        assertDoesNotThrow(() -> incomeRegister.addItem(income1));
        assertDoesNotThrow(() -> incomeRegister.addItem(income2));
    }

    @Nested
    @DisplayName("test getTotalSum and getIncomeByCategory methods")
    class testGetTotalSumAndGetIncomeByCategoryMethods {

        Income income1;
        Income income2;
        Income income3;

        @BeforeEach
        void addIncomeToRegister() {
            income1 = new Income("description1", 59.9f, false, IncomeCategory.SALARY, "03.03.23");
            income2 = new Income("description2", 62.4f, true, IncomeCategory.GIFT, "01.02.21");
            income3 = new Income("description3", 9.81f, false, IncomeCategory.SALARY, "05.07.23");
            incomeRegister.addItem(income1);
            incomeRegister.addItem(income2);
            incomeRegister.addItem(income3);
        }

        @Test
        @DisplayName("getTotalSum method gives correct amount")
        void getTotalSumCorrectAmount() {
            double totalIncome = 59.9f + 62.4f + 9.81f;
            assertEquals(Math.round(incomeRegister.getTotalSum()), Math.round(totalIncome));
        }

        @Test
        @DisplayName("getItemsByCategory gives expected Income back")
        void getIncomeByCategoryGivesCorrectIncome() {
            List<Income> incomeSalary = incomeRegister.getIncomeByCategory(IncomeCategory.SALARY);
            assertTrue(incomeSalary.contains(income1) && incomeSalary.contains(income3));
        }
    }
}

